#!/usr/bin/python2.3

import datetime
import xml.dom.minidom
import urllib

class WeatherForecast:
	def __init__(self, latitude=42.64645, longitude=-71.31418, days=5):
		# http://www.weather.gov/forecasts/xml/SOAP_server/ndfdSOAPclientByDay.php?lat=42.612635&lon=-71.320400&format=12+hourly&startDate=2006-07-14&numDays=4
		forecastURLbase = "http://www.weather.gov/forecasts/xml/SOAP_server/ndfdSOAPclientByDay.php?lat=%f&lon=%f&format=12+hourly&startDate=%s&numDays=%d"
		self.Forecast = {}
		self.daysList = []
		self.url = forecastURLbase % (
			latitude,
			longitude,
			datetime.date.today(),
			days)
		self.update()

	def getText(self, nodelist):
		rc = ""
		for node in nodelist:
			if node.nodeType == node.TEXT_NODE:
				rc = rc + node.data
		return rc

	def update(self):
		dom = xml.dom.minidom.parse(urllib.urlopen(self.url))
		layoutKeys = []
		days = {}
		temperatures = {}
		precip = []
		weather = []
		icons = []

		for timeLayout in dom.getElementsByTagName("time-layout"):
			layoutKey = self.getText(timeLayout.getElementsByTagName("layout-key")[0].childNodes)
			layoutKeys.append(layoutKey)
			days[layoutKey] = []
			for day in timeLayout.getElementsByTagName("start-valid-time"):
				days[layoutKey].append( day.getAttribute('period-name') )
				temperatures[layoutKey] = {}

		parameters = dom.getElementsByTagName("parameters")[0]

		for temperatureEl in parameters.getElementsByTagName("temperature"):
			layout = temperatureEl.getAttribute("time-layout")
			type = temperatureEl.getAttribute("type")
			temperatures[layout][type] = []
			for value in temperatureEl.getElementsByTagName("value"):
				temperatures[layout][type].append( self.getText(value.childNodes) )

		for precipEl in parameters.getElementsByTagName("probability-of-precipitation"):
			for value in precipEl.getElementsByTagName("value"):
				precip.append( self.getText(value.childNodes) )

		for weatherEl in parameters.getElementsByTagName("weather"):
			for value in weatherEl.getElementsByTagName("weather-conditions"):
				weather.append( value.getAttribute("weather-summary") )

		for iconEl in parameters.getElementsByTagName("conditions-icon"):
			for value in iconEl.getElementsByTagName("icon-link"):
				icons.append( self.getText(value.childNodes) )

		self.daysList = days[layoutKeys[0]]
		for x in range( 0, len(self.daysList), 1 ):
			curDay = self.daysList[x]
			self.Forecast[curDay] = {}
			self.Forecast[curDay]["MaxTemp"] = temperatures[layoutKeys[0]]["maximum"][x]
			self.Forecast[curDay]["MinTemp"] = temperatures[layoutKeys[1]]["minimum"][x]

		for x in range( 0, len(self.daysList)*2, 1 ):
			curDay = self.daysList[x/2]
			if x % 2 == 0:
				self.Forecast[curDay]["PrecipDay"] = precip[x]
				self.Forecast[curDay]["WeatherDay"] = weather[x]
				self.Forecast[curDay]["IconDay"] = icons[x]
			else:
				self.Forecast[curDay]["PrecipNight"] = precip[x]
				self.Forecast[curDay]["WeatherNight"] = weather[x]
				self.Forecast[curDay]["IconNight"] = icons[x]

		
	def test(self):
		for day in self.daysList:
			print day + " Min:" + self.Forecast[day]["MinTemp"] + " Max:" + self.Forecast[day]["MaxTemp"] + " Precip-Day:" + self.Forecast[day]["PrecipDay"] + " Precip-Night:" + self.Forecast[day]["PrecipNight"]
			print "\t" + self.Forecast[day]["WeatherDay"] + " ...... " + self.Forecast[day]["WeatherNight"]
			print "\t" + self.Forecast[day]["IconDay"] + " ...... " + self.Forecast[day]["IconNight"]
			print

